@@ -1460,9 +1460,11 @@ class Pile(Widget, WidgetContainerMixin, WidgetContainerListContentsMixin):
             elif f == GIVEN:
                 l.append(height)
                 remaining -= height
-            else:
+            elif height:
                 l.append(None)
                 wtotal += height
+            else:
+                l.append(0) # zero-weighted items treated as ('given', 0)
 
         if wtotal == 0:
             raise PileError, "No weighted widgets found for Pile treated as a box widget"
@@ -1644,6 +1646,8 @@ class Pile(Widget, WidgetContainerMixin, WidgetContainerListContentsMixin):
             if wrow + r > row:
                 break
             wrow += r
+        else:
+            return False
 
         focus = focus and self.focus_item == w
         if is_mouse_press(event) and button == 1:
@@ -2104,17 +2108,15 @@ class Columns(Widget, WidgetContainerMixin, WidgetContainerListContentsMixin):
         for i, (width, (w, options)) in enumerate(zip(widths, self.contents)):
             end = x + width
             if w.selectable():
-                # FIXME: sometimes, col == 'left' - that doesn't seem like its handled here, does it?
-                # assert isinstance(x, int) and isinstance(col, int), (x, col)
-                if x > col and best is None:
+                if col != RIGHT and (col == LEFT or x > col) and best is None:
                     # no other choice
                     best = i, x, end, w, options
                     break
-                if x > col and col-best[2] < x-col:
+                if col != RIGHT and x > col and col-best[2] < x-col:
                     # choose one on left
                     break
                 best = i, x, end, w, options
-                if col < end:
+                if col != RIGHT and col < end:
                     # choose this one
                     break
             x = end + self.dividechars
