@@ -68,7 +68,9 @@ class Screen(BaseScreen, RealTerminal):
         self._setup_G1_done = False
         self._rows_used = None
         self._cy = 0
-        self.bright_is_bold = os.environ.get('TERM',None) != "xterm"
+        term = os.environ.get('TERM', '')
+        self.bright_is_bold = not term.startswith("xterm")
+        self.back_color_erase = not term.startswith("screen")
         self._next_timeout = None
         self._term_output_file = sys.stdout
         self._term_input_file = sys.stdin
@@ -111,25 +113,32 @@ class Screen(BaseScreen, RealTerminal):
             os.write(self._resize_pipe_wr, B('R'))
         self._resized = True
         self.screen_buf = None
-      
+
+    def _sigcont_handler(self, signum, frame):
+        self.stop()
+        self.start()
+        self._sigwinch_handler(None, None)
+
     def signal_init(self):
         """
-        Called in the startup of run wrapper to set the SIGWINCH 
-        signal handler to self._sigwinch_handler.
+        Called in the startup of run wrapper to set the SIGWINCH
+        and SIGCONT signal handlers.
 
         Override this function to call from main thread in threaded
         applications.
         """
         signal.signal(signal.SIGWINCH, self._sigwinch_handler)
-    
+        signal.signal(signal.SIGCONT, self._sigcont_handler)
+
     def signal_restore(self):
         """
         Called in the finally block of run wrapper to restore the
-        SIGWINCH handler to the default handler.
+        SIGWINCH and SIGCONT signal handlers.
 
         Override this function to call from main thread in threaded
         applications.
         """
+        signal.signal(signal.SIGCONT, signal.SIG_DFL)
         signal.signal(signal.SIGWINCH, signal.SIG_DFL)
       
     def set_mouse_tracking(self):
@@ -163,7 +172,7 @@ class Screen(BaseScreen, RealTerminal):
     def start(self, alternate_buffer=True):
         """
         Initialize the screen and input mode.
-        
+
         alternate_buffer -- use alternate screen buffer
         """
         assert not self._started
@@ -182,13 +191,15 @@ class Screen(BaseScreen, RealTerminal):
         self._alternate_buffer = alternate_buffer
         self._input_iter = self._run_input_iter()
         self._next_timeout = self.max_wait
-        
+
         if not self._signal_keys_set:
             self._old_signal_keys = self.tty_signal_keys(fileno=fd)
 
         super(Screen, self).start()
 
-    
+        signals.emit_signal(self, INPUT_DESCRIPTORS_CHANGED)
+
+
     def stop(self):
         """
         Restore the screen.
@@ -196,6 +207,9 @@ class Screen(BaseScreen, RealTerminal):
         self.clear()
         if not self._started:
             return
+
+        signals.emit_signal(self, INPUT_DESCRIPTORS_CHANGED)
+
         self.signal_restore()
 
         fd = self._term_input_file.fileno()
@@ -237,7 +251,7 @@ class Screen(BaseScreen, RealTerminal):
             return fn()
         finally:
             self.stop()
-            
+
     def get_input(self, raw_keys=False):
         """Return pending input as a list.
 
@@ -251,39 +265,42 @@ class Screen(BaseScreen, RealTerminal):
         If raw_keys is False (default) this function will return a list
         of keys pressed.  If raw_keys is True this function will return
         a ( keys pressed, raw keycodes ) tuple instead.
-        
-        Examples of keys returned
-        -------------------------
-        ASCII printable characters:  " ", "a", "0", "A", "-", "/" 
-        ASCII control characters:  "tab", "enter"
-        Escape sequences:  "up", "page up", "home", "insert", "f1"
-        Key combinations:  "shift f1", "meta a", "ctrl b"
-        Window events:  "window resize"
-        
-        When a narrow encoding is not enabled
-        "Extended ASCII" characters:  "\\xa1", "\\xb2", "\\xfe"
 
-        When a wide encoding is enabled
-        Double-byte characters:  "\\xa1\\xea", "\\xb2\\xd4"
+        Examples of keys returned:
 
-        When utf8 encoding is enabled
-        Unicode characters: u"\\u00a5", u'\\u253c"
-        
-        Examples of mouse events returned
-        ---------------------------------
-        Mouse button press: ('mouse press', 1, 15, 13), 
-                            ('meta mouse press', 2, 17, 23)
-        Mouse drag: ('mouse drag', 1, 16, 13),
-                    ('mouse drag', 1, 17, 13),
-                ('ctrl mouse drag', 1, 18, 13)
-        Mouse button release: ('mouse release', 0, 18, 13),
-                              ('ctrl mouse release', 0, 17, 23)
+        * ASCII printable characters:  " ", "a", "0", "A", "-", "/" 
+        * ASCII control characters:  "tab", "enter"
+        * Escape sequences:  "up", "page up", "home", "insert", "f1"
+        * Key combinations:  "shift f1", "meta a", "ctrl b"
+        * Window events:  "window resize"
+
+        When a narrow encoding is not enabled:
+
+        * "Extended ASCII" characters:  "\\xa1", "\\xb2", "\\xfe"
+
+        When a wide encoding is enabled:
+
+        * Double-byte characters:  "\\xa1\\xea", "\\xb2\\xd4"
+
+        When utf8 encoding is enabled:
+
+        * Unicode characters: u"\\u00a5", u'\\u253c"
+
+        Examples of mouse events returned:
+
+        * Mouse button press: ('mouse press', 1, 15, 13), 
+                              ('meta mouse press', 2, 17, 23)
+        * Mouse drag: ('mouse drag', 1, 16, 13),
+                      ('mouse drag', 1, 17, 13),
+                      ('ctrl mouse drag', 1, 18, 13)
+        * Mouse button release: ('mouse release', 0, 18, 13),
+                                ('ctrl mouse release', 0, 17, 23)
         """
         assert self._started
-        
+
         self._wait_for_input_ready(self._next_timeout)
         self._next_timeout, keys, raw = self._input_iter.next()
-        
+
         # Avoid pegging CPU at 100% when slowly resizing
         if keys==['window resize'] and self.prev_input_resize:
             while True:
@@ -299,14 +316,14 @@ class Screen(BaseScreen, RealTerminal):
                     break
             if keys[-1:]!=['window resize']:
                 keys.append('window resize')
-                
+
         if keys==['window resize']:
             self.prev_input_resize = 2
         elif self.prev_input_resize == 2 and not keys:
             self.prev_input_resize = 1
         else:
             self.prev_input_resize = 0
-        
+
         if raw_keys:
             return keys, raw
         return keys
@@ -318,18 +335,21 @@ class Screen(BaseScreen, RealTerminal):
 
         Use this method if you are implementing yout own event loop.
         """
+        if not self._started:
+            return []
+
         fd_list = [self._term_input_file.fileno(), self._resize_pipe_rd]
         if self.gpm_mev is not None:
             fd_list.append(self.gpm_mev.stdout.fileno())
         return fd_list
-        
+
     def get_input_nonblocking(self):
         """
-        Return a (next_input_timeout, keys_pressed, raw_keycodes) 
+        Return a (next_input_timeout, keys_pressed, raw_keycodes)
         tuple.
 
         Use this method if you are implementing your own event loop.
-        
+
         When there is input waiting on one of the descriptors returned
         by get_input_descriptors() this method should be called to
         read and process the input.
@@ -337,8 +357,6 @@ class Screen(BaseScreen, RealTerminal):
         This method expects to be called in next_input_timeout seconds
         (a floating point number) if there is no input waiting.
         """
-        assert self._started
-
         return self._input_iter.next()
 
     def _run_input_iter(self):
@@ -615,22 +633,27 @@ class Screen(BaseScreen, RealTerminal):
                 continue
 
             sb.append(row)
-            
+
             # leave blank lines off display when we are using
             # the default screen buffer (allows partial screen)
             if partial_display() and y > self._rows_used:
                 if is_blank_row(row):
                     continue
                 self._rows_used = y
-            
+
             if y or partial_display():
                 o.append(set_cursor_position(0, y))
             # after updating the line we will be just over the
             # edge, but terminals still treat this as being
             # on the same line
-            cy = y 
-            
-            if y == maxrow-1:
+            cy = y
+
+            whitespace_at_end = False
+            if row and row[-1][2][-1:] == B(' ') and self.back_color_erase:
+                whitespace_at_end = True
+                a, cs, run = row[-1]
+                row = row[:-1] + [(a, cs, run.rstrip(B(' ')))]
+            elif y == maxrow-1 and maxcol>1:
                 row, back, ins = self._last_row(row)
 
             first = True
@@ -673,26 +696,23 @@ class Screen(BaseScreen, RealTerminal):
 
                 if cs == "U":
                     o.append(escape.IBMPC_OFF)
+            if whitespace_at_end:
+                o.append(escape.ERASE_IN_LINE_RIGHT)
 
         if r.cursor is not None:
             x,y = r.cursor
-            o += [set_cursor_position(x, y), 
+            o += [set_cursor_position(x, y),
                 escape.SHOW_CURSOR  ]
             self._cy = y
-        
-        if self._resized: 
+
+        if self._resized:
             # handle resize before trying to draw screen
             return
         try:
-            k = 0
             for l in o:
                 if isinstance(l, bytes) and PYTHON3:
                     l = l.decode('utf-8')
                 self._term_output_file.write(l)
-                k += len(l)
-                if k > 1024:
-                    self._term_output_file.flush()
-                    k = 0
             self._term_output_file.flush()
         except IOError, e:
             # ignore interrupted syscall
@@ -701,19 +721,19 @@ class Screen(BaseScreen, RealTerminal):
 
         self.screen_buf = sb
         self._screen_buf_canvas = r
-                
-    
+
+
     def _last_row(self, row):
         """On the last row we need to slide the bottom right character
         into place. Calculate the new line, attr and an insert sequence
         to do that.
-        
+
         eg. last row:
         XXXXXXXXXXXXXXXXXXXXYZ
-        
+
         Y will be drawn after Z, shifting Z into position.
         """
-        
+
         new_row = row[:-1]
         z_attr, z_cs, last_text = row[-1]
         last_cols = util.calc_width(last_text, 0, len(last_text))
